import sys
from collections import Counter
from copy import deepcopy
from datetime import timedelta

import pandas as pd
from numpy import unique

from ..common import general_utils
from ..common.nlp_utils import STTM, CleanText, Features, Polarity

ERR_SYS = "\nSystem error: "


class STTMgroupsTW:
    """
    This class groups texts into groups of similar topics.
    """

    def __init__(self, df_replies, account_ids=[], account_names=[]):
        """
        This method computes the DataFrame 'df_replies_full'
        which contains all the information of the twitter replies, including column 'group' and 'polarity'.

        Parameters
        ----------
        df_replies:
            type: DataFrame
            Information of the tweets or the twitter replies.
            This Pandas DataFrame must have columns 'text', 'twitter_id', 'screen_name',  and 'replying_to_id' and 'replying_to' if the data frame contains tweet replies.
        df_users:
            type: DataFrame
            Information of the twitter users.
        groups:
            type: dict
            Maps the groups (client, competition, archetype, trends) to the
            corresponding page ids for each group.
        """
        METHOD_NAME = "__init__"
        df_replies_full = deepcopy(df_replies)
        self.df_replies_full = df_replies_full
        # self.groups = groups
        try:
            if "replying_to" in self.df_replies_full.keys():
                self.df_replies_full["name"] = self.df_replies_full["replying_to"]

            else:
                self.df_replies_full["name"] = self.df_replies_full["screen_name"]
            self.df_replies_full["all"] = "all groups"

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            print(
                "Warning: Class: {self.__str__()}\nMethod: {METHOD_NAME}' no 'replying_to' or 'screen_name' column in DataFrame"
            )
            if "name" not in self.df_replies_full.keys():
                self.df_replies_full["name"] = "no name"

        if account_ids:
            self.df_replies_full = self.df_replies_full[
                self.df_replies_full.replying_to_id.isin(account_ids)
            ]
        elif account_names:
            self.df_replies_full = self.df_replies_full[
                self.df_replies_full.name.isin(account_names)
            ]

        try:
            if df_replies.empty:
                print("Warning: input data DataFrame is empty.")
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_replies_full = pd.DataFrame(columns=[""])

    def remove_from_text(self, text, toremove):
        text_copy = deepcopy(text)
        for word in text:
            if word in toremove:
                text_copy.remove(word)
        return text_copy

    def get_sttm_groups(self, with_remove=False, list_remove=[], min_texts=100):
        METHOD_NAME = "get_sttm_groups"

        try:

            # cleaning text:
            if "processed_text" not in self.df_replies_full.keys():
                self.df_replies_full["processed_text"] = self.df_replies_full[
                    "text"
                ].apply(
                    lambda msg: CleanText(msg).process_text(
                        mentions=True, hashtags=True, links=True, spec_chars=True
                    )
                )

            # drop empty replies
            self.df_replies_full = self.df_replies_full.dropna(
                subset=["processed_text"]
            )
            self.df_replies_full = self.df_replies_full.drop(
                self.df_replies_full[self.df_replies_full["processed_text"] == ""].index
            )

            if list_remove:
                # self.df_replies_full['processed_text'] = self.df_replies_full['processed_text'].(lambda x: json.loads(x))
                self.df_replies_full["processed_text"] = self.df_replies_full[
                    "processed_text"
                ].apply(lambda x: str(x).split(" "))
                self.df_replies_full["processed_text"] = self.df_replies_full[
                    "processed_text"
                ].apply(lambda x: self.remove_from_text(x, list_remove))
                self.df_replies_full["processed_text"] = self.df_replies_full[
                    "processed_text"
                ].apply(lambda msg: str(msg))

            # getting the polarity of the clean text
            if "polarity" not in self.df_replies_full.keys():
                self.df_replies_full = Polarity().polarity(df_text=self.df_replies_full)

            # tokenized text:
            if "tokenized_text" not in self.df_replies_full.keys():
                getfeatures = Features()
                self.df_replies_full[
                    "tokenized_text"
                ] = self.df_replies_full.processed_text.apply(
                    lambda msg: getfeatures.pos_tags(msg)["words"]
                )

            # drop empty comments
            self.df_replies_full = self.df_replies_full.dropna(
                subset=["tokenized_text"]
            )
            self.df_replies_full = self.df_replies_full.drop(
                self.df_replies_full[self.df_replies_full["tokenized_text"] == ""].index
            )
            self.df_replies_full[
                "len_text"
            ] = self.df_replies_full.tokenized_text.apply(lambda msg: len(msg))
            self.df_replies_full = self.df_replies_full[
                self.df_replies_full["len_text"] != 0
            ]

            # getting sttm groups
            if self.df_replies_full.tokenized_text.shape[0] > min_texts:
                self.df_replies_full = STTM(self.df_replies_full).sttm_model()
            else:
                print(
                    "warning: in",
                    f"Class: {self.__str__()}\nMethod: {METHOD_NAME}",
                    "not enough texts in DataFrame for STTM grouping. Returning value 1 for group column for sttm_group",
                )
                self.df_replies_full["sttm_group"] = 1

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            if "polarity" not in self.df_replies_full.keys():
                self.df_replies_full["polarity"] = ""
            if "sttm_group" not in self.df_replies_full.keys():
                self.df_replies_full["sttm_group"] = ""

    def get_sttm_groups_withremove(
        self, df_preprocessed_sttm, list_toremove=[], min_texts=100
    ):
        METHOD_NAME = "get_sttm_groups_withremove"
        self.df_preprocessed_sttm = deepcopy(df_preprocessed_sttm)

        try:
            # self.df_preprocessed_sttm['tokenized_text'] = self.df_preprocessed_sttm['tokenized_text'].apply(lambda x: json.loads(x))
            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: x[2:-2])
            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: str(x).split("', '"))

            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: self.remove_from_text(x, list_toremove))

            # drop empty comments
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.dropna(
                subset=["tokenized_text"]
            )
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.drop(
                self.df_preprocessed_sttm[
                    self.df_preprocessed_sttm["tokenized_text"] == ""
                ].index
            )
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.drop(
                self.df_preprocessed_sttm[
                    self.df_preprocessed_sttm["tokenized_text"] == " "
                ].index
            )
            self.df_preprocessed_sttm[
                "len_text"
            ] = self.df_preprocessed_sttm.tokenized_text.apply(lambda msg: len(msg))
            self.df_preprocessed_sttm = self.df_preprocessed_sttm[
                self.df_preprocessed_sttm["len_text"] != 0
            ]

            # getting sttm groups
            if self.df_preprocessed_sttm.tokenized_text.shape[0] > min_texts:
                self.df_preprocessed_sttm = STTM(self.df_preprocessed_sttm).sttm_model()
            else:
                print(
                    "warning: in",
                    f"Class: {self.__str__()}\nMethod: {METHOD_NAME}",
                    "\n Not enough texts in DataFrame for STTM grouping. Returning value 1 for group column for sttm_group",
                )
                self.df_preprocessed_sttm["sttm_group"] = 1

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_preprocessed_sttm["sttm_group"] = ""

    def counter_of_words(self, words, mention, n_words=10):
        """
        Counts the elements in the list 'words' and returns
        the 'n_words' most frequent.

        Parameters
        ----------
        words
            type: list
            List of words to count.
        mention:
            type: str
            Mention to delete from the counter to avoid overcounting
            mentions in responses to a certain account.
        n_words:
            type: int
            Number of most frequent words to return

        Returns
        -------
        dict
        """

        counter = Counter(words)
        try:
            del counter["@" + mention.lower()]
        except Exception:
            pass
        return dict(counter.most_common(n_words))

    def get_percentage(self, counter):
        """
        Computes the frequency (percentage) of the elements in the
        dictionary 'counter'.

        Parameters
        ----------
        counter
            type: dict
            Dictionary of elements to count.

        Returns
        -------
        dict
        """

        percentage = Counter(counter)
        percentage = dict(
            [
                (i, perc / sum(percentage.values()) * 100.0)
                for i, perc in percentage.most_common()
            ]
        )
        return percentage

    def message_to_group_columns(self):
        """
        This function constructs the columns to group the dataframe
        'data' for audiencies visualizations.

        Returns
        -------
        DataFrame
        """

        data_texts = deepcopy(self.df_replies_full)

        if "__date_cot" not in data_texts.keys():
            data_texts["__date_cot"] = pd.to_datetime(
                data_texts["created_at"]
            ) - timedelta(hours=5)

        if "replying_to" in data_texts.keys():
            data_texts = data_texts[
                [
                    "sttm_group",
                    "tokenized_text",
                    "replying_to",
                    "__date_cot",
                    "polarity",
                ]
            ]
            data_texts = data_texts.rename(columns={"replying_to": "__mention"})
        else:
            data_texts = data_texts[
                ["sttm_group", "tokenized_text", "__date_cot", "polarity"]
            ]
            data_texts["__mention"] = ""

        self.data_texts = data_texts

    def get_grouped_dataframe(self):
        """
        Groups the DataFrame 'data' into a DataFrame containing information
        abput the groups of texts.

        Parameters
        ----------
        data
            type: DataFrame
            DataFrame containing information about the texts.

        Returns
        -------
        DataFrame
        """
        data_grouped = deepcopy(
            self.data_texts[
                ["sttm_group", "tokenized_text", "__mention", "__date_cot", "polarity"]
            ]
        )

        data_grouped["polarity"] = data_grouped["polarity"].apply(lambda pol: str(pol))

        data_grouped["__weekday"] = data_grouped["__date_cot"].apply(
            lambda d: d.strftime("%A")
        )

        data_grouped["__n_docs"] = 1

        data_grouped = (
            data_grouped[
                [
                    "sttm_group",
                    "__n_docs",
                    "tokenized_text",
                    "__mention",
                    "__weekday",
                    "polarity",
                ]
            ]
            .groupby("sttm_group")
            .agg(
                {
                    "__n_docs": "count",
                    "tokenized_text": "sum",
                    "__mention": "last",
                    "__weekday": lambda x: ",".join(x)
                    if len(unique(data_grouped["__weekday"])) > 1
                    else "",
                    "polarity": lambda x: ",".join(x),
                }
            )
        )

        data_grouped["__percentage"] = (
            data_grouped["__n_docs"] * 100 / sum(data_grouped["__n_docs"])
        )

        data_grouped = data_grouped.rename(columns={"tokenized_text": "__words"})

        data_grouped["__words"] = data_grouped.apply(
            lambda row: self.counter_of_words(row["__words"], row["__mention"]), axis=1
        )
        data_grouped = data_grouped.drop(columns=["__mention"])

        data_grouped["__weekday"] = data_grouped["__weekday"].apply(
            lambda wd: wd.split(",")
        )
        data_grouped = data_grouped.rename(columns={"__weekday": "__presence"})
        data_grouped["__presence"] = data_grouped["__presence"].apply(
            lambda p: self.get_percentage(p)
        )

        data_grouped["polarity"] = data_grouped["polarity"].apply(
            lambda wd: wd.split(",")
        )
        data_grouped = data_grouped.rename(
            columns={"polarity": "__polarity_distribution"}
        )
        data_grouped["__polarity_distribution"] = data_grouped[
            "__polarity_distribution"
        ].apply(lambda p: self.get_percentage(p))

        data_grouped = data_grouped.reset_index().sort_values(
            "__percentage", ascending=False
        )

        return data_grouped



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 28 21:26:06 2020

@author: oscar
"""

import re as reg
import traceback
from copy import deepcopy
from sys import exc_info

import pandas as pd

ERR_SYS = "system error: "


def thread_mention(df_media_tw):
    """
    thread_mention this function filters the twitter threads

    Parameters
    ----------
    df_media_tw : TYPE dataframe
        DESCRIPTION. dataframe with mentios

    Returns
    -------
    df_media_tw : TYPE dataframe
        DESCRIPTION. dataframe with mentiosbut filter threads

    """
    if len(df_media_tw) > 2:
        thread = []
        i_t = 0
        f_t = 1
        while i_t < len(df_media_tw):
            if df_media_tw.in_reply_to_status_id.iloc[i_t] == None:
                thread.append(0)
            else:
                if (
                    float(df_media_tw.in_reply_to_status_id.iloc[i_t])
                    / float(df_media_tw.tweet_id.iloc[f_t])
                    == 1
                ):
                    thread.append(1)
                else:
                    thread.append(0)
            i_t += 1
            if f_t < len(df_media_tw) - 1:
                f_t += 1
        df_media_tw["thread"] = thread
        df_media_tw = df_media_tw[df_media_tw.thread == 0]
        df_media_tw = df_media_tw.drop(columns=["thread"])
        df_media_tw = df_media_tw.sort_values(["tweet_id", "created_at"])

    return df_media_tw


def create_regex(list_terms):
    """
    Parameters
    ----------
    list_terms : TYPE list
        DESCRIPTION. list with the text to apply

    Returns
    -------
    pat : TYPE list
        DESCRIPTION. lislist without @name

    """
    pattern = r"\b@*" + r"\b|\b@*".join(list_terms) + r"\b"
    pat = reg.compile(pattern)
    return pat


class SocialpresTW:
    def __init__(self, df_tw, df_count, brand, auto_n=True, thread=True):
        """Digital social presence
        This function calculates social presence, comparing it with the
        weight of each digital media

        Parameters
        ----------
        df_ig : TYPE dataframe
            DESCRIPTION. Contains the information collected from social media
            in Instagram
        df_count : TYPE dataframe
            DESCRIPTION. Contains the media information to consult
        media_weights : TYPE dict
            DESCRIPTION.Weights and names the digital media
        brand : TYPE string
            DESCRIPTION. brand analyze
        auto_n : TYPE bolean
            DESCRIPTION. remove mentions of the same account if true
        thread : TYPE bolean
            DESCRIPTION. remove mentions of thread twitter if true

        Returns
        -------
        None.

        """

        self.df_tw = df_tw
        self.df_count = df_count
        self.brand = brand
        self.auto_n = auto_n
        self.thread = thread

    def c_pres_tw(self):
        """Twitter media presence
        calculates Twitter only number of appearances

        Returns
        -------
        df_med_c
        TYPE dataframe
            DESCRIPTION. each count number of appearances in Twitter

        """
        col_med = ["created_at", "description", "text", "tweet_id", "screen_name"]

        df_tw = self.df_tw
        df_c = self.df_count
        auto_n = self.auto_n
        df_empty_c = pd.DataFrame(columns=["media", "mentions"])
        method_name = "c_pres_tw"

        try:
            df_tw = df_tw[col_med]
            df_tw = deepcopy(df_tw)
            df_tw.created_at = pd.to_datetime(df_tw.created_at)

            df_tw.text = df_tw.text.apply(lambda x: str(x))
            df_tw.text = df_tw.text.apply(lambda x: x.lower())

            df_m = deepcopy(df_tw)
            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["text"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[5:-1]
                for col in col_names:
                    for index in range(len(df_m)):
                        temp_1 = df_m[col]
                        if (temp_1.iloc[index]) & (df_m.screen_name.iloc[index] == col):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_c = df_m2.drop(
                columns=["created_at", "description", "text", "tweet_id", "auto"]
            ).groupby("screen_name")
            df_med_c = df_med_c.sum().T

            df_med_c = pd.DataFrame(df_med_c.T.sum())
            df_med_c = df_med_c.reset_index()
            df_med_c = df_med_c.rename(columns={0: "mentions", "index": "media"})
            df_med_c = df_med_c.sort_values("mentions", ascending=False)
            df_med_c = df_med_c.reset_index(drop=True)

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_c = df_empty_c

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_c = df_empty_c

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_c = df_empty_c

        return df_med_c

    def id_pres_tw(self):
        """Twitter media presence
        calculates Twitter only number of appearances

        Returns
        -------
        df_med_id
        TYPE dataframe
            DESCRIPTION. each count number of appearances in Twitter

        """
        col_med = [
            "created_at",
            "description",
            "text",
            "tweet_id",
            "in_reply_to_status_id",
            "screen_name",
        ]

        df_tw = self.df_tw
        df_c = self.df_count
        brand = self.brand
        auto_n = self.auto_n
        thread = self.thread
        df_empty_id = pd.DataFrame(
            columns=["created_at", "text", "screen_name", "tweet_id"]
        )
        method_name = "id_pres_tw"

        try:
            df_tw = df_tw[col_med]
            df_tw = deepcopy(df_tw)
            df_tw.created_at = pd.to_datetime(df_tw.created_at)

            df_tw.text = df_tw.text.apply(lambda x: str(x))
            df_tw.text = df_tw.text.apply(lambda x: x.lower())

            df_m = deepcopy(df_tw)
            for idd, row in df_c.set_index("nombre").iterrows():
                if idd:
                    account_terms_regex = create_regex(
                        [t.strip() for t in row.terminos.split(",")]
                    )
                    df_m[f"{idd}"] = df_m["text"].str.contains(
                        account_terms_regex, regex=True
                    )

            df_m["auto"] = 0

            if auto_n:
                col_names = df_m.columns[6:-1]
                for col in col_names:
                    for index in range(len(df_m)):
                        temp_1 = df_m[col]
                        if (temp_1.iloc[index]) & (df_m.screen_name.iloc[index] == col):
                            df_m.auto.iloc[index] = 1
                df_m2 = df_m[df_m.auto == 0]
                df_m2 = df_m2.reset_index(drop=True)
            else:
                df_m2 = df_m

            df_med_id = df_m2[
                [
                    "created_at",
                    "text",
                    "screen_name",
                    "tweet_id",
                    "in_reply_to_status_id",
                    brand,
                ]
            ]
            df_med_id = df_med_id[df_med_id[brand]]
            df_med_id = df_med_id.drop(columns=brand)
            df_med_id = df_med_id.reset_index(drop=True)

            if thread:
                df_med_id_th = thread_mention(df_med_id)
                df_med_id_th = df_med_id_th.drop(columns="in_reply_to_status_id")
            else:
                df_med_id_th = deepcopy(df_med_id)
                df_med_id_th = df_med_id_th.drop(columns="in_reply_to_status_id")

        except KeyError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_id_th = df_empty_id

        except AttributeError as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_id_th = df_empty_id

        except Exception as e_3:
            print(e_3)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            traceback.print_exc()

            df_med_id_th = df_empty_id

        return df_med_id_th



import json
import re
import sys
from copy import deepcopy
from datetime import timedelta

import pandas as pd

from ..common import general_utils, metric_transformation

ERR_SYS = "\nSystem error: "


class EngagementRateTW:
    def __init__(self, df_tweets, df_replies, groups, mode="status", remove_rt=True):
        """
        This method computes the DataFrame 'df_tweets_full' with the column 'reply_count_no_api'
        which is a count of the replies for every post.

        Parameters
        ----------
        df_tweets:
            type: DataFrame
            this Pandas DataFrame muts have columns
            'tweet_id'.
        df_replies:
            type: DataFrame
            this Pandas DataFrame muts have columns
            'text', 'in_reply_to_status_id'.
        groups:
            type: dict
            Maps the groups (client, compentition, archetype, trends) to the
            corresponding page ids for each group.
        mode:
            type: str
            Selects the module to compute the engagement rates for.
            default = 'status'
            If set to 'terms' df_replies and groups are no used.
        """

        METHOD_NAME = "__init__"

        REPLIES_COLUMNS = ["text", "in_reply_to_status_id"]
        TWEETS_COLUMNS = [
            "twitter_id",
            "screen_name",
            "created_at",
            "tweet_id",
            "favorite_count",
            "retweet_count",
            "ac_followers_count",
            "in_reply_to_status_id",
            "media_entities",
            "user_mentions",
            "hashtags",
            "text",
            "profile_image",
        ]
        OUTPUT_COLUMNS = [
            "reply_count_no_api",
            "twitter_id",
            "screen_name",
            "group",
            "created_at",
            "tweet_id",
            "ac_followers_count",
            "favorite_count",
            "retweet_count",
            "in_reply_to_status_id",
            "media_entities",
            "user_mentions",
            "hashtags",
            "text",
            "profile_image",
        ]
        self.remove_rt = remove_rt
        self.mode = mode

        if mode == "status":
            if len(df_tweets) > 0 and len(df_replies) > 0:
                try:
                    df_tweets_full = deepcopy(
                        df_tweets[df_tweets["in_reply_to_status_id"].isna()][
                            TWEETS_COLUMNS
                        ]
                    )
                    df_tweets_full = df_tweets_full.drop_duplicates(
                        subset=[
                            "screen_name",
                            "created_at",
                            "user_mentions",
                            "hashtags",
                            "text",
                        ],
                        keep=False,
                    )
                    df_replies_count = (
                        df_replies[REPLIES_COLUMNS]
                        .groupby("in_reply_to_status_id")
                        .count()
                        .reset_index()
                    )
                    df_replies_count = df_replies_count.rename(
                        columns={
                            "in_reply_to_status_id": "tweet_id",
                            "text": "reply_count_no_api",
                        }
                    )
                    df_tweets_full = pd.merge(
                        df_tweets_full, df_replies_count, on=["tweet_id"], how="left"
                    )
                    df_tweets_full["reply_count_no_api"] = df_tweets_full[
                        "reply_count_no_api"
                    ].fillna(0)
                    df_tweets_full["group"] = df_tweets_full["twitter_id"].apply(
                        lambda tid: general_utils.get_group(tid, groups)
                    )
                    df_tweets_full["created_at"] = pd.to_datetime(
                        df_tweets_full["created_at"]
                    ) - timedelta(hours=5)
                    self.df_tweets_full = df_tweets_full

                except Exception as e:
                    exception_type = sys.exc_info()[0]
                    print(ERR_SYS + str(exception_type))
                    print(e)
                    print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                    self.df_tweets_full = pd.DataFrame(
                        columns=OUTPUT_COLUMNS, dtype=object
                    )

            else:
                print("Warning: One of the DataFrames is empty. It cannot be computed.")
                self.df_tweets_full = pd.DataFrame(columns=OUTPUT_COLUMNS, dtype=object)

        elif mode == "terms":
            try:
                df_tweets_full = deepcopy(df_tweets[TWEETS_COLUMNS])
                df_tweets_full = df_tweets_full.drop_duplicates(
                    subset=[
                        "screen_name",
                        "created_at",
                        "user_mentions",
                        "hashtags",
                        "text",
                        "media_entities",
                    ],
                    keep=False,
                )
                df_tweets_full[
                    "reply_count_no_api"
                ] = 0  # set replies to zero, temporary until official API
                df_tweets_full["group"] = "terms"  # set group to 'terms'
                df_tweets_full = df_tweets_full[
                    df_tweets_full["ac_followers_count"] != 0
                ]  # Avoiding division by zero for users with zero followes.
                df_tweets_full["created_at"] = pd.to_datetime(
                    df_tweets_full["created_at"]
                ) - timedelta(hours=5)
                if remove_rt:
                    rt_flag_list, unique_text_list, author_name_list = [], [], []
                    rt_pattern = re.compile(r"^(?:RT|rt) \@[a-zA-Z0-9\-\_]+\:\s")
                    for index, row in df_tweets_full.iterrows():
                        findings = re.findall(rt_pattern, row["text"])
                        rt_flag = 0
                        unique_text, author_name = row["text"], row["screen_name"]
                        if len(findings) > 0:
                            rt_flag = 1
                            unique_text = re.sub(rt_pattern, "", row["text"])
                            author_name = re.sub(
                                re.compile(r"^(?:RT|rt) \@"), "", findings[0]
                            ).replace(": ", "")
                        rt_flag_list.append(rt_flag)
                        unique_text_list.append(unique_text.lower())
                        author_name_list.append(author_name)
                    df_tweets_full["rt_flag"] = rt_flag_list
                    df_tweets_full["unique_text"] = unique_text_list
                    df_tweets_full["author_name"] = author_name_list
                self.df_tweets_full = df_tweets_full

            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                self.df_tweets_full = pd.DataFrame(columns=OUTPUT_COLUMNS, dtype=object)

        else:
            raise RuntimeError(
                f'Mode {mode} is not available. Modes available: "status" and "terms".'
            )

    def by_reach(self):
        """
        This method computes the engagement rate for every post based on the
        reach of the post. It stores it on the column
        'engagement_rate_by_reach' on the input Pandas DataFrame 'df_tweets_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_reach"

        df_tweets_full = self.df_tweets_full
        try:
            df_tweets_full["engagement_rate_by_reach"] = df_tweets_full.apply(
                lambda row: 100
                * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                / row.reach_count,
                axis=1,
            )
            return df_tweets_full.dropna(subset=["engagement_rate_by_reach"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_tweets_full.columns) + ["engagement_rate_by_reach"]
            )

    def by_post(self):
        """
        This method computes the engagement rate for every post based on the
        number of followers of the account. It stores it on the column
        'engagement_rate_by_post' on the input Pandas DataFrame 'df_tweets_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_post"

        df_tweets_full = self.df_tweets_full
        try:
            if self.mode == "terms":
                df_tweets_full["engagement_rate_by_post"] = df_tweets_full.apply(
                    lambda row: row.favorite_count
                    + row.retweet_count
                    + row.reply_count_no_api,
                    axis=1,
                )

                if self.remove_rt:
                    df_tweets_max_er = df_tweets_full[
                        ["screen_name", "unique_text", "engagement_rate_by_post"]
                    ]
                    df_tweets_max_er = (
                        df_tweets_max_er.groupby(["screen_name", "unique_text"])
                        .max()
                        .reset_index()
                    )
                    df_tweets_full = df_tweets_full.merge(
                        df_tweets_max_er,
                        left_on=[
                            "author_name",
                            "unique_text",
                            "engagement_rate_by_post",
                        ],
                        right_on=[
                            "screen_name",
                            "unique_text",
                            "engagement_rate_by_post",
                        ],
                        how="inner",
                    )
                    df_tweets_full = df_tweets_full.drop(
                        ["rt_flag", "unique_text", "author_name", "screen_name_y"],
                        axis=1,
                    )
                    df_tweets_full = df_tweets_full.rename(
                        columns={"screen_name_x": "screen_name"}
                    )

            else:
                df_tweets_full["engagement_rate_by_post"] = df_tweets_full.apply(
                    lambda row: 100
                    * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                    / row.ac_followers_count,
                    axis=1,
                )

            METRIC = "engagement_rate_by_post"
            ITEM_COLUMN = "twitter_id"
            df_tweets_full = df_tweets_full.sort_values(by=["created_at"])
            df_tweets_full = metric_transformation.MetricCategorization(
                df_tweets_full, METRIC, ITEM_COLUMN
            ).categorize()

            return df_tweets_full.dropna(subset=["engagement_rate_by_post"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_tweets_full.columns)
                + ["engagement_rate_by_post", "boundary", "rel_engagement_rate_by_post"]
            )

    def by_impressions(self):
        """
        This method computes the engagement rate for every post based on the
        impressions of the post. It stores it on the column
        'engagement_rate_by_impressions' on the input Pandas DataFrame 'df_tweets_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_impressions"

        df_tweets_full = self.df_tweets_full
        try:
            df_tweets_full["engagement_rate_by_impressions"] = df_tweets_full.apply(
                lambda row: 100
                * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                / row.impressions_count,
                axis=1,
            )
            return df_tweets_full.dropna(subset=["engagement_rate_by_impressions"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_tweets_full.columns)
                + ["engagement_rate_by_impressions"]
            )

    def by_day(self, grouped=True):
        """
        This method computes the engagement rate by day for each account
        based on the number of followers. It stores it on the column
        'engagement_rate_by_impressions' on the output Pandas DataFrame 'df_eng_by_day'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account by day.
            If True returns the DataFrame by group by day, default=True.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_day"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["twitter_id", "screen_name"]

        df_tweets_full = self.df_tweets_full
        try:
            df_eng_by_day = deepcopy(df_tweets_full)

            df_eng_by_day["date"] = df_eng_by_day.created_at.apply(lambda d: d.date)

            df_eng_by_day = (
                df_eng_by_day[
                    [
                        "twitter_id",
                        "screen_name",
                        "group",
                        "date",
                        "retweet_count",
                        "reply_count_no_api",
                        "favorite_count",
                        "ac_followers_count",
                    ]
                ]
                .groupby(["twitter_id", "screen_name", "date"])
                .agg(
                    {
                        "group": "last",
                        "retweet_count": "sum",
                        "favorite_count": "sum",
                        "reply_count_no_api": "sum",
                        "ac_followers_count": "max",
                    }
                )
            )
            df_eng_by_day["engagement_rate_by_day"] = df_eng_by_day.apply(
                lambda row: 100
                * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                / row.ac_followers_count,
                axis=1,
            )
            df_eng_by_day = df_eng_by_day.reset_index()

            if grouped:
                df_eng_by_day = (
                    df_eng_by_day[["group", "date", "engagement_rate_by_day"]]
                    .groupby(["group", "date"])
                    .mean()
                )
                df_eng_by_day = df_eng_by_day.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_day = df_eng_by_day.rename(
                    columns={"twitter_id": "_object_id", "screen_name": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_day"
            df_eng_by_day = df_eng_by_day.sort_values(by=["date"])
            df_eng_by_day = metric_transformation.MetricCategorization(
                df_eng_by_day, METRIC, ITEM_COLUMN
            ).categorize()

            return df_eng_by_day[
                PAGE_COLUMNS
                + [
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                ]
            ].dropna(subset=["engagement_rate_by_day"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                ]
            )

    def by_views(self):
        """
        This method computes the engagement rate for every video post based on the
        views. It stores it on the column 'engagement_rate_by_post'
        on the output Pandas DataFrame 'df_eng_by_views'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_views"

        df_tweets_full = self.df_tweets_full
        try:
            df_eng_by_views = deepcopy(df_tweets_full)
            df_eng_by_views = df_eng_by_views[~df_eng_by_views.media_entities.isna()]
            if len(df_eng_by_views) > 0:
                df_eng_by_views.media_entities = df_eng_by_views.media_entities.apply(
                    lambda ent: json.loads(ent)
                )

                df_eng_by_views["engagement_rate_by_views"] = df_eng_by_views.apply(
                    lambda row: 100
                    * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                    / row.view_count
                    if row.media_entities[0]["type"] == "video"
                    else None,
                    axis=1,
                )
                return df_eng_by_views.dropna(subset=["engagement_rate_by_views"])
            else:
                print("There are no tweets with videos.")

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_tweets_full.columns) + ["engagement_rate_by_views"]
            )

    def by_accounts(self, grouped=True):
        """
        This method computes the engagement rate by page
        based on the number of followers. It stores it on the column
        'engagement_rate_by_accounts' on the output Pandas DataFrame 'df_eng_by_accounts'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account.
            If True returns the DataFrame by group, default=True

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_accounts"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["twitter_id", "screen_name"]

        df_tweets_full = self.df_tweets_full
        try:
            df_eng_by_accounts = deepcopy(df_tweets_full)

            df_eng_by_accounts = (
                df_eng_by_accounts[
                    [
                        "twitter_id",
                        "screen_name",
                        "group",
                        "retweet_count",
                        "reply_count_no_api",
                        "favorite_count",
                        "ac_followers_count",
                    ]
                ]
                .groupby(["twitter_id", "screen_name"])
                .agg(
                    {
                        "group": "last",
                        "retweet_count": "sum",
                        "favorite_count": "sum",
                        "reply_count_no_api": "sum",
                        "ac_followers_count": "max",
                    }
                )
            )
            df_eng_by_accounts[
                "engagement_rate_by_accounts"
            ] = df_eng_by_accounts.apply(
                lambda row: 100
                * (row.favorite_count + row.retweet_count + row.reply_count_no_api)
                / row.ac_followers_count,
                axis=1,
            )
            df_eng_by_accounts = df_eng_by_accounts.reset_index()

            if grouped:
                df_eng_by_accounts = (
                    df_eng_by_accounts[["group", "engagement_rate_by_accounts"]]
                    .groupby(["group"])
                    .mean()
                )
                df_eng_by_accounts = df_eng_by_accounts.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_accounts = df_eng_by_accounts.rename(
                    columns={"twitter_id": "_object_id", "screen_name": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_accounts"
            df_eng_by_accounts = df_eng_by_accounts.sort_values(by=["group"])
            df_eng_by_accounts = metric_transformation.MetricCategorization(
                df_eng_by_accounts, METRIC, ITEM_COLUMN
            ).categorize()

            return df_eng_by_accounts[
                PAGE_COLUMNS
                + [
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ]
            ].dropna(subset=["engagement_rate_by_accounts"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ],
                dtype=object,
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug 18 20:44:09 2020

@author: Joan-Felipe Mendoza
"""
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None

ERR_SYS = "System error: "


class TacTips:
    def __init__(
        self,
        df_s,
        dim,
        item_col,
        item_name,
        met="engagement_rate_by_post",
        terms_size=5,
    ):
        """
        These functions deliver tac-tips based on the content of a pre-calculated dataframe (df_s),
        considering a met (metric) and the dimension to be analyzed.

        Parameters
        ----------
        df_s:
            type: Pandas DataFrame
            Pre-calculated dataframe containing the data needed to calculate the tac-tips
            (including metrics and dimensions).
        dim:
            type: string
            Name of dimension ('hashtag', 'word', 'mention', 'most_effective_pages', 'day', 'hour')
        item_col:
            type: string
            Name of the column that contains the item_name
        item_name:
            type: string
            Name of the item to analyze (in case of group: 'trends', 'competitors', 'no_group', 'brand', 'archetypes')
        met:
            type: string
            Name of the column within df_s containing the metric.
        terms_size:
            type: integer
            Number of terms to be included within the tac-tip
            (for HT, mentions, words, or most effective pages)
        """
        self.df_s = df_s
        self.met = met
        self.terms_size = terms_size
        self.dim = dim
        self.item_col = item_col
        self.item_name = item_name

    def get_tactip_values(self):
        """
        Returns
        -------
        TYPE string
            DESCRIPTION. values for tactip message
        """
        method_name = "get_tactip_values"

        try:
            df_item = self.df_s[self.df_s[self.item_col] == self.item_name]

            if self.dim in ["hashtag", "word", "mention"]:
                terms = df_item.nlargest(self.terms_size, self.met)[self.dim].tolist()
                dim_v1 = ", ".join(terms[:-1])
                dim_v2 = terms[-1]
                return dim_v1, dim_v2

            if self.dim == "most_effective_pages":
                df_pages = (
                    df_item[["page_name", self.met]].groupby(["page_name"]).mean()
                )
                df_pages = df_pages.reset_index()
                terms = df_pages.nlargest(self.terms_size, self.met)[
                    "page_name"
                ].tolist()
                dim_v1 = ", ".join(terms[:-1])
                dim_v2 = terms[-1]
                return dim_v1, dim_v2

            if self.dim in ["day", "hour"]:
                metric_avg = df_item[self.met].mean()
                df_item["perc_diff_vs_avg"] = (
                    df_item[self.met] - metric_avg
                ) / metric_avg
                df_item["perc_diff_vs_avg"] = df_item["perc_diff_vs_avg"] * 100
                best_m, best_p, temp_1 = (
                    df_item.nlargest(1, self.met)[
                        [self.dim, self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                max_i = int(round(temp_1, 0))
                worst_m, worst_p, temp_2 = (
                    df_item.nsmallest(1, self.met)[
                        [self.dim, self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                min_i = int(round(temp_2, 0))
                return best_m, best_p, max_i, worst_m, worst_p, min_i

            if self.dim in ["perc_audience", "diff_audience"]:
                df_item = df_item.rename(columns={"date": self.dim})
                metric_avg = df_item[self.met].mean()
                df_item["perc_diff_vs_avg"] = (
                    df_item[self.met] - metric_avg
                ) / metric_avg
                df_item["perc_diff_vs_avg"] = df_item["perc_diff_vs_avg"] * 100
                best_m, best_p, temp_1 = (
                    df_item.nlargest(1, self.met)[
                        [self.dim, self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                max_i = int(round(temp_1, 0))
                worst_m, worst_p, temp_2 = (
                    df_item.nsmallest(1, self.met)[
                        [self.dim, self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                min_i = int(round(temp_2, 0))
                return best_m, best_p, max_i, worst_m, worst_p, min_i

            if self.dim in ["moment_of_day"]:
                metric_avg = df_item[self.met].mean()
                df_item["perc_diff_vs_avg"] = (
                    df_item[self.met] - metric_avg
                ) / metric_avg
                df_item["perc_diff_vs_avg"] = df_item["perc_diff_vs_avg"] * 100
                best_d, best_h, best_p, temp_1 = (
                    df_item.nlargest(1, self.met)[
                        ["day", "hour", self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                max_i = int(round(temp_1, 0))
                worst_d, worst_h, worst_p, temp_2 = (
                    df_item.nsmallest(1, self.met)[
                        ["day", "hour", self.met, "perc_diff_vs_avg"]
                    ]
                    .values[0]
                    .tolist()
                )
                min_i = int(round(temp_2, 0))
                return best_d, best_h, best_p, max_i, worst_d, worst_h, worst_p, min_i

        except TypeError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            values = None
            if self.dim in ["hashtag", "word", "mention", "most_effective_pages"]:
                values = [None] * 2
            if self.dim in ["day", "hour", "perc_audience", "diff_audience"]:
                values = [None] * 6
            if self.dim in ["moment_of_day"]:
                values = [None] * 8
            return values

        except Exception as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")
            values = None
            if self.dim in ["hashtag", "word", "mention", "most_effective_pages"]:
                values = [None] * 2
            if self.dim in ["day", "hour", "perc_audience", "diff_audience"]:
                values = [None] * 6
            if self.dim in ["moment_of_day"]:
                values = [None] * 8
            return values



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jul 26 09:07:01 2020

@author: oscar
"""

from copy import deepcopy
from sys import exc_info

import pandas as pd
import statsmodels.api as sm

IMP = "page_posts_impressions_paid_unique"
POS = "page_positive_feedback_by_type_unique"
ERR_SYS = "System error: "


class PerfImprPos:
    def __init__(self, df_r, lim_sdt=True, lim_sdt_p=1, lim_s=0, lim_i=0):
        """
        Impressions Performance Vs positive feedback
        this class calculates the performance of the impressions in a period,
        considering the data backwards

        Parameters
        ----------
        df_r : TYPE dataframe
            DESCRIPTION. dataframe with information from facebook insights, should
            have page_posts_impressions_paid_unique and page_positive_feedback_by_type_unique
        lim_sdt : TYPE, boolean
            DESCRIPTION. The default is true. if the limit is the standard deviation
        lim_sdt_p : TYPE, float
            DESCRIPTION. The default is 1. standard deviation adjustment
        lim_s : TYPE, float
            DESCRIPTION. The default is 0. manual upper limit value if lim_sdt is false
        lim_i : TYPE, float
            DESCRIPTION. The default is 0. manual lower limit value if lim_sdt is false

        Returns
        -------
        None.

        """
        self.df_r = df_r
        self.lim_sdt = lim_sdt
        self.lim_sdt_p = lim_sdt_p
        self.lim_s = lim_s
        self.lim_i = lim_i

    def perf_imp_pos(self):
        """
        Impressions Performance Vs positive feedback
        This function calculates the performance of the impressions in a period,
        considering the data backwards

        Returns
        -------
        perf : TYPE str
            DESCRIPTION. less: low performance,
            higher: hight perfomance,
            ok: expected performance
        y_real : TYPE float
            DESCRIPTION. value of real data page_positive_feedback_by_type_unique
        y_pred : TYPE float
            DESCRIPTION. value of predicted data page_positive_feedback_by_type_unique
        df_mod : TYPE dataframe optional
            DESCRIPTION. regression data
        data : TYPE dataframe
            DESCRIPTION. data calculation

        """

        df_r = deepcopy(self.df_r)
        lim_sdt = self.lim_sdt
        lim_sdt_p = self.lim_sdt_p
        lim_s = self.lim_s
        lim_i = self.lim_i

        var_empty = None
        method_name = "perf_imp_pos"

        df_r = df_r[(df_r.name == IMP) | (df_r.name == POS)]
        df_r = df_r[df_r.period == "day"]
        df_r.end_time = pd.to_datetime(df_r.end_time)

        try:
            df_group = df_r.groupby(["end_time", "name"], as_index=False).sum()
            df_mod = df_group[df_group.end_time != max(df_group.end_time)]

            imp_m = list(df_mod.value[df_mod.name == IMP])
            pos_m = list(df_mod.value[df_mod.name == POS])
            df_reg = pd.DataFrame({"imp": imp_m, "pos": pos_m})

            x_m = df_reg["imp"]
            x_m = sm.add_constant(x_m)
            y_m = df_reg["pos"]

            mod = sm.OLS(y_m, x_m).fit()

            data = df_group[df_group.end_time == max(df_group.end_time)]

            if lim_sdt:
                lim_sup = mod.fittedvalues.std() * lim_sdt_p
                lim_inf = -mod.fittedvalues.std() * lim_sdt_p
            else:
                lim_sup = lim_s
                lim_inf = lim_i

            x_imp = data[data.name == IMP].value.iloc[0]
            y_pred = (mod.params[1] * x_imp) + mod.params[0]

            y_real = data[data.name == POS].value.iloc[0]

            if y_real < lim_inf + y_pred:
                perf = "less"
            elif y_real > lim_sup + y_pred:
                perf = "higher"
            else:
                perf = "ok"

        except AttributeError as e_1:
            print(e_1)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            perf = var_empty
            y_real = var_empty
            y_pred = var_empty

        except Exception as e_2:
            print(e_2)
            print(ERR_SYS + str(exc_info()[0]))
            print(f"Class: {self.__str__()}\nMethod: {method_name}")

            perf = var_empty
            y_real = var_empty
            y_pred = var_empty

        return perf, y_real, y_pred



import sys
from copy import deepcopy
from datetime import timedelta

import pandas as pd

from ..common import general_utils, metric_transformation

ERR_SYS = "\nSystem error: "


def get_name(pid, dict_page_id_to_name):
    try:
        out_name = dict_page_id_to_name[pid]
    except Exception:
        out_name = "no_name"
    return out_name


class EngagementRateFB:
    def __init__(self, df_posts, df_pages, groups):
        """
        This method computes the DataFrame 'df_posts_full' with the columns
        'group' and 'fan_count' which is a count of the fans in the day the post was made.

        Parameters
        ----------
        df_posts:
            type: DataFrame
            this Pandas DataFrame must have columns
            'created_time' and 'page_id'.
        df_pages:
            type: DataFrame
            this Pandas DataFrame must have columns
            'fan_count', 'page_id' and 'created_at'.
        groups:
            type: dict
            Maps the groups (client, competition, archetype, trends) to the
            corresponding page ids for each group.
        """

        METHOD_NAME = "__init__"

        PAGES_COLUMNS = ["fan_count", "page_id", "created_at"]
        POSTS_COLUMNS = [
            "page_id",
            "created_time",
            "post_id",
            "likes",
            "reactions_love",
            "reactions_wow",
            "reactions_haha",
            "reactions_sad",
            "reactions_angry",
            "reactions_thankful",
            "shares",
            "comments",
            "message",
            "permalink_url",
            "type",
            "message_tags",
        ]
        OUTPUT_COLUMNS = [
            "fan_count",
            "page_id",
            "page_name",
            "date",
            "group",
            "created_time",
            "post_id",
            "likes",
            "reactions_love",
            "reactions_wow",
            "reactions_haha",
            "reactions_sad",
            "reactions_angry",
            "reactions_thankful",
            "shares",
            "comments",
            "message",
            "permalink_url",
            "type",
            "message_tags",
        ]

        if len(df_posts) > 0 and len(df_pages) > 0:
            try:
                df_fan_count = (
                    df_pages[PAGES_COLUMNS]
                    .groupby(["page_id", "created_at"])
                    .last()
                    .reset_index()
                )

                df_fan_count["created_at"] = pd.to_datetime(
                    df_fan_count["created_at"], format="%Y-%m-%dT%H:%M:%S"
                )
                df_fan_count["created_at"] = pd.to_datetime(
                    df_fan_count["created_at"]
                ) - timedelta(hours=5)
                df_fan_count["date"] = df_fan_count["created_at"].apply(
                    lambda d: d.date
                )
                df_fan_count = df_fan_count.drop(columns=["created_at"])

                df_posts_full = deepcopy(df_posts[POSTS_COLUMNS])

                page_id_name_fb = {}
                for idd, row in df_pages.iterrows():
                    page_id_name_fb[row.page_id] = row["name"]
                df_posts_full["page_name"] = df_posts_full.page_id.apply(
                    lambda pid: get_name(pid, page_id_name_fb)
                )

                df_posts_full["created_time"] = pd.to_datetime(
                    df_posts_full["created_time"]
                ) - timedelta(hours=5)
                df_posts_full["date"] = df_posts_full["created_time"].apply(
                    lambda d: d.date
                )
                df_posts_full = pd.merge(
                    df_posts_full, df_fan_count, on=["page_id", "date"], how="left"
                )
                df_posts_full["fan_count"] = df_posts_full.sort_values(
                    ["page_id", "date"]
                )["fan_count"].fillna(method="bfill")

                df_posts_full["group"] = df_posts_full["page_id"].apply(
                    lambda pid: general_utils.get_group(pid, groups)
                )

                self.df_posts_full = df_posts_full

            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS, dtype=object)

        else:
            print("Warning: One of the DataFrames is empty. It cannot be computed.")
            self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS, dtype=object)

    def by_reach(self):
        """
        This method computes the engagement rate for every post based on the
        reach of the post. It stores it on the column
        'engagement_rate_by_reach' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_reach"

        df_posts_full = self.df_posts_full
        try:
            df_posts_full["engagement_rate_by_reach"] = df_posts_full.apply(
                lambda row: 100
                * (
                    row.likes
                    + row.reactions_love
                    + row.reactions_wow
                    + row.reactions_haha
                    + row.reactions_sad
                    + row.reactions_angry
                    + row.reactions_thankful
                    + row.shares
                    + row.comments
                )
                / row.reach_count,
                axis=1,
            )
            return df_posts_full.dropna(subset=["engagement_rate_by_reach"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns) + ["engagement_rate_by_reach"]
            )

    def by_post(self):
        """
        This method computes the engagement rate for every post based on the
        number of followers of the account. It stores it on the column
        'engagement_rate_by_post' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_post"

        df_posts_full = self.df_posts_full
        try:
            df_posts_full["engagement_rate_by_post"] = df_posts_full.apply(
                lambda row: 100
                * (
                    row.likes
                    + row.reactions_love
                    + row.reactions_wow
                    + row.reactions_haha
                    + row.reactions_sad
                    + row.reactions_angry
                    + row.reactions_thankful
                    + row.shares
                    + row.comments
                )
                / row.fan_count,
                axis=1,
            )

            METRIC = "engagement_rate_by_post"
            ITEM_COLUMN = "page_id"
            df_posts_full = df_posts_full.sort_values(by=["created_time"])
            df_posts_full = metric_transformation.MetricCategorization(
                df_posts_full, METRIC, ITEM_COLUMN
            ).categorize()

            return df_posts_full.dropna(subset=["engagement_rate_by_post"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns)
                + ["engagement_rate_by_post", "boundary", "rel_engagement_rate_by_post"]
            )

    def by_impressions(self):
        """
        This method computes the engagement rate for every post based on the
        impressions of the post. It stores it on the column
        'engagement_rate_by_impressions' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_impressions"

        df_posts_full = self.df_posts_full
        try:
            df_posts_full["engagement_rate_by_impressions"] = df_posts_full.apply(
                lambda row: 100
                * (
                    row.likes
                    + row.reactions_love
                    + row.reactions_wow
                    + row.reactions_haha
                    + row.reactions_sad
                    + row.reactions_angry
                    + row.reactions_thankful
                    + row.shares
                    + row.comments
                )
                / row.impressions_count,
                axis=1,
            )
            return df_posts_full.dropna(subset=["engagement_rate_by_impressions"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns) + ["engagement_rate_by_impressions"]
            )

    def by_day(self, grouped=True):
        """
        This method computes the engagement rate by day for each account
        based on the number of followers. It stores it on the column
        'engagement_rate_by_impressions' on the output Pandas DataFrame 'df_eng_by_day'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account by day.
            If True returns the DataFrame by group by day, default=True

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_day"

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_day = deepcopy(df_posts_full)

            df_eng_by_day = (
                df_eng_by_day[
                    [
                        "page_id",
                        "page_name",
                        "group",
                        "date",
                        "shares",
                        "comments",
                        "likes",
                        "reactions_love",
                        "reactions_wow",
                        "reactions_haha",
                        "reactions_sad",
                        "reactions_angry",
                        "reactions_thankful",
                        "fan_count",
                    ]
                ]
                .groupby(["page_id", "page_name", "date"])
                .agg(
                    {
                        "group": "last",
                        "shares": "sum",
                        "comments": "sum",
                        "likes": "sum",
                        "reactions_love": "sum",
                        "reactions_wow": "sum",
                        "reactions_haha": "sum",
                        "reactions_sad": "sum",
                        "reactions_angry": "sum",
                        "reactions_thankful": "sum",
                        "fan_count": "max",
                    }
                )
            )
            df_eng_by_day["engagement_rate_by_day"] = df_eng_by_day.apply(
                lambda row: 100
                * (
                    row.likes
                    + row.reactions_love
                    + row.reactions_wow
                    + row.reactions_haha
                    + row.reactions_sad
                    + row.reactions_angry
                    + row.reactions_thankful
                    + row.shares
                    + row.comments
                )
                / row.fan_count,
                axis=1,
            )
            df_eng_by_day = df_eng_by_day.reset_index()

            if grouped:
                df_eng_by_day = (
                    df_eng_by_day[["group", "date", "engagement_rate_by_day"]]
                    .groupby(["group", "date"])
                    .mean()
                )
                df_eng_by_day = df_eng_by_day.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_day = df_eng_by_day.rename(
                    columns={"page_id": "_object_id", "page_name": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_day"
            df_eng_by_day = df_eng_by_day.sort_values(by=["date"])
            df_eng_by_day = metric_transformation.MetricCategorization(
                df_eng_by_day, METRIC, ITEM_COLUMN
            ).categorize()

            return df_eng_by_day[
                PAGE_COLUMNS
                + [
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                ]
            ].dropna(subset=["engagement_rate_by_day"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                ]
            )

    def by_views(self):
        """
        This method computes the engagement rate for every video post based on the
        views. It stores it on the column 'engagement_rate_by_post'
        on the output Pandas DataFrame 'df_eng_by_views'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_views"

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_views = deepcopy(df_posts_full)
            df_eng_by_views = df_eng_by_views[df_eng_by_views.type == "video"]
            if len(df_eng_by_views) > 0:
                df_eng_by_views["engagement_rate_by_views"] = df_eng_by_views.apply(
                    lambda row: 100
                    * (
                        row.likes
                        + row.reactions_love
                        + row.reactions_wow
                        + row.reactions_haha
                        + row.reactions_sad
                        + row.reactions_angry
                        + row.reactions_thankful
                        + row.shares
                        + row.comments
                    )
                    / row.view_count,
                    axis=1,
                )
                return df_eng_by_views.dropna(subset=["engagement_rate_by_views"])
            else:
                print("There are no posts with videos.")
                return pd.DataFrame(
                    columns=list(df_posts_full.columns) + ["engagement_rate_by_views"]
                )

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns) + ["engagement_rate_by_views"]
            )

    def by_accounts(self, grouped=True):
        """
        This method computes the engagement rate by page
        based on the number of followers. It stores it on the column
        'engagement_rate_by_accounts' on the output Pandas DataFrame 'df_eng_by_accounts'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account.
            If True returns the DataFrame by group, default=True

        Returns
        -------
        DataFrame
        """
        METHOD_NAME = "by_accounts"

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_accounts = deepcopy(df_posts_full)

            df_eng_by_accounts = (
                df_eng_by_accounts[
                    [
                        "page_id",
                        "page_name",
                        "group",
                        "shares",
                        "comments",
                        "likes",
                        "reactions_love",
                        "reactions_wow",
                        "reactions_haha",
                        "reactions_sad",
                        "reactions_angry",
                        "reactions_thankful",
                        "fan_count",
                        "post_id",
                    ]
                ]
                .groupby(["page_id", "page_name"])
                .agg(
                    {
                        "group": "last",
                        "shares": "sum",
                        "comments": "sum",
                        "likes": "sum",
                        "reactions_love": "sum",
                        "reactions_wow": "sum",
                        "reactions_haha": "sum",
                        "reactions_sad": "sum",
                        "reactions_angry": "sum",
                        "reactions_thankful": "sum",
                        "fan_count": "max",
                    }
                )
            )
            df_eng_by_accounts[
                "engagement_rate_by_accounts"
            ] = df_eng_by_accounts.apply(
                lambda row: 100
                * (
                    row.likes
                    + row.reactions_love
                    + row.reactions_wow
                    + row.reactions_haha
                    + row.reactions_sad
                    + row.reactions_angry
                    + row.reactions_thankful
                    + row.shares
                    + row.comments
                )
                / row.fan_count,
                axis=1,
            )
            df_eng_by_accounts = df_eng_by_accounts.reset_index()

            if grouped:
                df_eng_by_accounts = (
                    df_eng_by_accounts[["group", "engagement_rate_by_accounts"]]
                    .groupby(["group"])
                    .mean()
                )
                df_eng_by_accounts = df_eng_by_accounts.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_accounts = df_eng_by_accounts.rename(
                    columns={"page_id": "_object_id", "page_name": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_accounts"
            df_eng_by_accounts = df_eng_by_accounts.sort_values(by=["group"])
            df_eng_by_accounts = metric_transformation.MetricCategorization(
                df_eng_by_accounts, METRIC, ITEM_COLUMN
            ).categorize()

            return df_eng_by_accounts[
                PAGE_COLUMNS
                + [
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ]
            ].dropna(subset=["engagement_rate_by_accounts"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ],
                dtype=object,
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wj_analysis',
 'wj_analysis.common',
 'wj_analysis.digital_med',
 'wj_analysis.facebook',
 'wj_analysis.facebook_insig',
 'wj_analysis.instagram',
 'wj_analysis.reach_fb_ig',
 'wj_analysis.twitter',
 'wj_analysis.unit_test']

package_data = \
{'': ['*']}

install_requires = \
['numpy==1.17.4', 'pandas==0.25.3']

setup_kwargs = {
    'name': 'wj-analysis',
    'version': '0.3.10',
    'description': 'Whale&Jaguar Libary - Analysis',
    'long_description': None,
    'author': 'Sebastian Franco',
    'author_email': 'jsfranco@whaleandjaguar.co',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>3.7.1,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug 27 08:37:50 2020

@author: oscar
"""

from copy import deepcopy

import matplotlib.pyplot as plt
import pandas as pd

# from wj_analysis.common.nlp_utils import Polarity as pol_dist
import plotly.graph_objects as go

# import plotly.express as px
from numpy import arange, mean
from textblob import TextBlob

pd.options.mode.chained_assignment = None

FOLDER = "/home/oscar/Labs/Data_social_media/Pruebas PWJ-772 digital-social/Media/"
COLUMN = "text"
WORD = input("Marca a buscar: ")
WORDS = [WORD]

DF_ORG = pd.read_csv(FOLDER + "df_media.csv", index_col=0)

DF_ORG.date_published = pd.to_datetime(DF_ORG.date_published)

DF_ORG[COLUMN] = DF_ORG[COLUMN].apply(lambda x: str(x))
DF_ORG[COLUMN] = DF_ORG[COLUMN].apply(lambda x: x.lower())


def words_filter(df_w, words):
    # filtra la palabra objetivo
    DF_M = deepcopy(df_w)
    DF_M["word"] = None

    for i in range(len(DF_M)):
        for word in words:
            word = word.lower()
            if str(DF_M.text.iloc[i]).find(str(word)) > 0:
                DF_M.word.iloc[i] = "ok"
            else:
                continue

    DF_M_FILTER = DF_M[DF_M.word == "ok"]
    return DF_M_FILTER


DF_FILTER_1 = words_filter(df_w=DF_ORG, words=WORDS)


def find_point(df_p, carac="."):
    # filtra por puntos o el caracter escogido
    DF_P = deepcopy(df_p)
    Position = []
    for i in range(len(df_p)):
        pos = []
        last = 0
        text_c = str(DF_P.text.iloc[i])
        while text_c.find(carac, last) != -1:
            last = text_c.find(carac, last) + 1
            pos.append(last)
        Position.append(pos)
    return Position


DF_FILTER_1["Pos"] = find_point(df_p=DF_FILTER_1, carac=".")

for i in range(len(DF_FILTER_1)):
    plt.plot(
        range(len(DF_FILTER_1.Pos.iloc[i])), DF_FILTER_1.Pos.iloc[i], "o-", label=str(i)
    )
plt.title("Longitud de los textos palabra " + str(WORDS[0]))
plt.legend()
plt.show()

# parte entre puntos
TEXT = int(input("Texto a consultar: "))
text = TEXT
num_caract = 50
text_split = []
first = 0
phrases = []
for i in DF_FILTER_1.Pos.iloc[text]:
    p = DF_FILTER_1.text.iloc[text][first:i]
    long = i - first
    phrases.append(long)
    if long > num_caract:
        text_split.append(p)
    else:
        continue
    first = i

DF_POL = pd.DataFrame(text_split)
DF_POL["title"] = DF_FILTER_1.title.iloc[text]
DF_POL = DF_POL.rename(columns={0: "text"})

pol = []
mention = []
size_point = []
for i in range(len(DF_POL)):
    if str(DF_POL.text.iloc[i]).find(str(WORD)) > 0:
        mention.append(WORD)
        size_point.append(60)
    else:
        mention.append(0)
        size_point.append(20)
    get_pol = TextBlob(DF_POL.text.iloc[i])
    # polarity = get_pol.translate(to='en')
    polarity = get_pol.polarity
    pol.append(polarity)

# pol = pol_dist().polarity(df_text=text_split)

# plt.plot(range(len(pol)), pol, c="pink")
# for i in range(len(pol)):
#     plt.scatter(i, pol[i], s=len(text_split[i]) / 2, c=len(text_split[i]), alpha=0.8)
# plt.title(str(DF_FILTER_1.title.iloc[text]))
# plt.xlabel("numero de la frase")
# plt.ylabel("Polaridad")
# plt.grid()
# plt.show()

len_text = []
scale = mean(phrases) / 5
for i in range(len(text_split)):
    len_text.append(len(text_split[i]) / scale)

num_phrase = arange(0, len(pol))

# tamaño longitud frase
fig = go.Figure(
    data=[
        go.Scatter(
            x=num_phrase,
            y=pol,
            text=text_split,
            mode="markers",
            marker=dict(
                color=len_text,
                size=size_point,
                showscale=True,
                line=dict(width=2, color="MediumPurple"),
                colorbar=dict(
                    title="Tamaño frase",
                    tickvals=[min(len_text), mean(len_text), max(len_text)],
                    ticktext=["Bajo", "Promedio", "Alto"],
                ),
                colorscale="Blues",
            ),
        )
    ]
)

fig.update_traces(mode="markers+lines")
fig.update_layout(
    title=str(
        DF_FILTER_1.title.iloc[text]
        + ", tamaño total texto "
        + str(round(sum(len_text) * scale, 0))
    ),
    xaxis=dict(title="Frase"),
    yaxis=dict(
        title="Polaridad, "
        + "(promedio "
        + str(round(mean(pol), 2))
        + ")"
        + " (rango "
        + str(round((max(pol) - min(pol)), 2))
        + ")"
    ),
)
fig.show()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  3 17:50:42 2021

@author: oscar
"""
import random
import unittest
from copy import deepcopy

import pandas as pd

import wj_analysis.instagram.engagement_rates as IG

pd.options.mode.chained_assignment = None

groups_ig = {}

FOLDER = "D:/whale_jaguar/datos_redes_sociales/Informes_020421/Instagram/"

df_pages = pd.read_csv(FOLDER + "instagram_lib_profile.csv", low_memory=False)
df_posts = pd.read_csv(FOLDER + "instagram_lib_posts.csv", low_memory=False)

print("ut_effec_moments_ig.py")
print("post = " + str(len(df_posts)))
print("brands = " + str(len(df_posts.owner_id.unique())))
print("================================================")


class Test(unittest.TestCase):
    def setUp(self):
        """
        Variables to use in tests

        Returns
        -------
        None.

        """
        erfb = IG.EngagementRateIG(df_posts, df_pages, groups=groups_ig)
        self.df_ef_post = erfb.by_post()
        self.df_ef_account = erfb.by_accounts()
        self.df_pages_empty = pd.DataFrame(columns=df_pages.columns, dtype=object)
        self.df_posts_empty = pd.DataFrame(columns=df_posts.columns, dtype=object)

        brands = df_posts.groupby("owner_username", as_index=False).agg(
            {"owner_id": "last"}
        )
        self.brands = brands.to_dict(orient="index")
        self.brand_sample = random.sample(list(range(len(brands))), 3)

    def test_data_normal(self):
        """
        This test with data ok

        Returns
        -------
        None.

        """
        print("TEST_DATA_NORMAL...")

        self.assertGreater(len(self.df_ef_account), 0)

        print("================================================")

    def test_data_empty(self):
        """
        This test with data empty

        Returns
        -------
        None.

        """
        print("TEST_DATA_EMPTY...")

        erfb_empty = IG.EngagementRateIG(
            self.df_posts_empty, self.df_pages_empty, groups=groups_ig
        )
        df_ef_account = erfb_empty.by_accounts()
        self.assertAlmostEqual(len(df_ef_account), 0)

        print("================================================")

    def test_nan_data(self):
        """
        This test with data 20% null

        Returns
        -------
        None.

        """
        print("TEST_NAN_DATA...")

        print("nan in pages")
        df_pages_null = deepcopy(df_pages)
        array_ef = random.sample(
            list(range(len(df_pages_null))), len(df_pages_null) // 5
        )
        for i in array_ef:
            df_pages_null.date.iloc[i] = float("nan")
            df_pages_null.followers.iloc[i] = float("nan")

        erfb = IG.EngagementRateIG(df_posts, df_pages_null, groups=groups_ig)
        df_ef_acc_pa = erfb.by_accounts()

        self.assertGreater(len(df_ef_acc_pa), 0)

        print("nan in posts")
        df_post_null = deepcopy(df_posts)
        array_ef = random.sample(list(range(len(df_post_null))), len(df_post_null) // 5)
        for i in array_ef:
            # df_post_null.date_utc.iloc[i] = float("nan")
            df_post_null.owner_id.iloc[i] = float("nan")

        erfb = IG.EngagementRateIG(df_post_null, df_pages, groups=groups_ig)
        df_ef_acc_pa = erfb.by_accounts()

        self.assertGreater(len(df_ef_acc_pa), 0)

        print("nan in posts and pages")
        erfb = IG.EngagementRateIG(df_post_null, df_pages_null, groups=groups_ig)
        df_ef_acc_pa = erfb.by_accounts()

        self.assertGreater(len(df_ef_acc_pa), 0)

        print("================================================")

    def test_brands(self):
        """
        Test with data from three brands chosen at random

        Returns
        -------
        None.

        """
        print("TEST_BRANDS...")
        brand1 = self.brands[self.brand_sample[0]]
        brand2 = self.brands[self.brand_sample[1]]
        brand3 = self.brands[self.brand_sample[2]]

        print(brand1["owner_username"])
        df_pages_brand1 = df_pages[df_pages.userid == brand1["owner_id"]]
        df_posts_brand1 = df_posts[df_posts.owner_id == brand1["owner_id"]]
        erfb_brand1 = IG.EngagementRateIG(
            df_posts_brand1, df_pages_brand1, groups=groups_ig
        )
        df_ef_acc_brand1 = erfb_brand1.by_accounts()

        self.assertGreater(len(df_ef_acc_brand1), 0)

        print(brand2["owner_username"])
        df_pages_brand2 = df_pages[df_pages.userid == brand2["owner_id"]]
        df_posts_brand2 = df_posts[df_posts.owner_id == brand2["owner_id"]]
        erfb_brand2 = IG.EngagementRateIG(
            df_posts_brand2, df_pages_brand2, groups=groups_ig
        )
        df_ef_acc_brand2 = erfb_brand2.by_accounts()

        self.assertGreater(len(df_ef_acc_brand2), 0)

        print(brand3["owner_username"])
        df_pages_brand3 = df_pages[df_pages.userid == brand3["owner_id"]]
        df_posts_brand3 = df_posts[df_posts.owner_id == brand3["owner_id"]]
        erfb_brand3 = IG.EngagementRateIG(
            df_posts_brand3, df_pages_brand3, groups=groups_ig
        )
        df_ef_acc_brand3 = erfb_brand3.by_accounts()

        self.assertGreater(len(df_ef_acc_brand3), 0)

        print("================================================")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python 3.6.9
Created on Tue 2020-07-31

@author: Felipe Gomez-Cortes
@gitlab: felipegomez_wj
@mail: felipeg@whaleandjaguar.co

TODO: Handling corrupted dataframe (wrong data)
TODO: Add window time for the analysis (initial date, end date)
TODO: Add max_age parameter
TODO: Make the script lexible to cover

"""

from copy import deepcopy

import numpy as np
import pandas as pd

ERR_SYS = "\nSystem error: "


class PostEngagementRateSensor:
    """
    This class computes the average performance on different features (clicks,
    normalized clicks, engagement_fb) and their derivatives (delta_feature)
    of a set of posts, and returns when some of them are deviated
    from the average with an performance indicator (good, average, poor). The
    limits of the average behavior can be manual or automaticly established.
    """

    def __init__(self, df_posts):
        """
        Generates an instance of PostEngagementRateSensor after checking the
        columns and size of the input DataFrame.

        Parameters:
        -----------
        df_posts:
            type DataFrame
            Daily update information of the posts.
            This Pandas DataFrame must have columns 'post_id',
            'created_time', 'snapshot_time', 'post_clicks' and
            'post_impressions'
        """

        METHOD_NAME = "__init__"

        EXPECTED_COLUMNS = [
            "post_id",
            "created_time",
            "snapshot_time",
            "post_clicks",
            "post_impressions",
            "post_impressions_organic",
            "post_impressions_viral",
            "post_impressions_paid",
        ]
        EXPECTED_COLUMNS.sort()

        df = df_posts.dropna()

        if len(df) > 0:
            df_columns = list(df.columns.values)
            df_columns.sort()
            EXPECTED_COLUMNS.sort()
            if df_columns != EXPECTED_COLUMNS:
                print(
                    METHOD_NAME,
                    "\nWarning: the DataFrame has not the expected columns.",
                )
        else:
            print(
                METHOD_NAME,
                "\nWarning: the DataFrames is empty. It cannot be computed.",
            )
        # TODO: handling corrupted data.
        self.df = df

    def load_sensor(self):
        """
        This function computes the normalized clicks, and the derivative (delta)
        of post_clicks, post_clicks_normalized and efficiency_fb
        """
        METHOD_NAME = "load_sensor"

        df = self.df

        try:
            # convert strings to datetime formant
            # df["created_time"] = df["created_time"].apply(lambda _: pd.to_datetime(_) )
            # df["snapshot_time"] = df["snapshot_time"].apply(lambda _ : pd.to_datetime(_))
            df.loc[:, "created_time"] = deepcopy(
                pd.to_datetime(df.loc[:, "created_time"]).copy()
            )
            df.loc[:, "snapshot_time"] = deepcopy(
                pd.to_datetime(df.loc[:, "snapshot_time"])
            )

            # sort posts with this specific order to make easier to calculate
            # derivatives
            df = df.sort_values(by=["created_time", "post_id", "snapshot_time"])

            # define maximum of clicks to normalize clicks.
            post_id_list = df.post_id.unique()
            for i in range(len(post_id_list)):
                post_id = post_id_list[i]
                index = df["post_id"] == post_id
                Y = df.loc[index, "post_clicks"]
                post_clicks_max = Y.max()
                df.loc[index, "post_clicks_max"] = post_clicks_max
            df["effectivity_fb"] = 100 * df["post_clicks"] / df["post_impressions"]
            df["post_clicks_normalized"] = df["post_clicks"] / df["post_clicks_max"]

            # Derivatives
            # computes the derivative of the feature using
            # delta_feature = ( feature_today - feature_yesterday) / 1_day
            # Requieres the dataframe sorted by "created time", "post_id" and
            # "snapshot_time" in that specific order.
            delta_clicks = []
            delta_effectivity = []
            for i in range(len(df.index)):
                idx = df.index[i]
                idx_prev = df.index[i - 1]

                # First DataFrame element, or first register of the post.
                if (i == 0) or (df.loc[idx, "post_id"] != df.loc[idx_prev, "post_id"]):
                    delta_clicks.append(np.nan)
                    delta_effectivity.append(np.nan)

                # Compute Delta if it is a regular post with today and yesterday's data.
                elif i != 0:
                    delta_clicks.append(
                        df.loc[idx, "post_clicks"] - df.loc[idx_prev, "post_clicks"]
                    )
                    delta_effectivity.append(
                        df.loc[idx, "effectivity_fb"]
                        - df.loc[idx_prev, "effectivity_fb"]
                    )

            df["delta_clicks"] = delta_clicks
            df["delta_clicks_normalized"] = df["delta_clicks"] / df["post_clicks_max"]
            df["delta_effectivity_fb"] = delta_effectivity

            df["post_age_days"] = (df["snapshot_time"] - df["created_time"]).dt.days

            self.df = df
        except Exception as e:
            print(e)
            print(METHOD_NAME)

    def time_series(self, feature="post_clicks"):
        return self.df[[feature, "post_id", "snapshot_time", "post_age_days"]]

    def performance_indicator(
        self, feature="effectivity_fb", stat="std", upp_lim=0, low_lim=0, beta=1
    ):
        """
        Parameters:
        feature: Select one feature to compute the performance. It may be one
            from the list [post_clicks, post_clicks_normalized,
            effectivity_fb, delta_post_clicks, delta_post_clicks_normalized
            delt_effectivity_fb]
        stat: Defines how to define what is the 'average performance' of a post:
            'std': using the standard deviation and a beta multiplier
            'quartiles': using the first and third quartiles
            'manual': select the limits by hand.
        beta: (float) If stat='std', selects
            upp_lim = mean + beta * standard_deviation
            low_lim = mean - beta * standard_deviation
        upp_lim: (float) manual upper limit, only if stat="manual"
        low_lim: (float) manual lower limit, only if stat="manual

        Returns:
        DataFrame with the columns:
            created_time: (datetime) Post created time
            post_age_days: (int) self explanatory
            feature: (float) the value of the feature selected by the user
            perf_upp_lim: (float) performance upper limit as defined by the user
            perf_mean. (float) Mean performance calculated for same age posts
            perf_low_lim: (float) performance lower limit as defined by the user
            performance: (str) [good, average, poor]
        """

        df = self.df
        ages = range(0, 6)  # TODO: manual selection of the age period.

        df["performance"] = "average"
        df["perf_mean"] = np.nan
        df["perf_upp_lim"] = np.nan
        df["perf_low_lim"] = np.nan

        for age in ages:
            index = df["post_age_days"] == age

            series = df.loc[index, feature]
            series_mean = series.mean()
            series_std = series.std()

            if stat == "std":
                upp_lim = series_mean + beta * series_std
                low_lim = series_mean - beta * series_std
            if stat == "quartiles":
                upp_lim = series.quantile(0.75)
                low_lim = series.quantile(0.25)

            df.loc[index, "perf_mean"] = series_mean
            df.loc[index, "perf_upp_lim"] = upp_lim
            df.loc[index, "perf_low_lim"] = low_lim

        idx = df[feature] == np.nan
        df.loc[idx, "performance"] = np.nan

        idx = df[feature] < df["perf_low_lim"]
        df.loc[idx, "performance"] = "poor"

        idx = df[feature] > df["perf_upp_lim"]
        df.loc[idx, "performance"] = "good"

        return df.loc[
            :,
            [
                "created_time",
                "post_age_days",
                feature,
                "perf_upp_lim",
                "perf_mean",
                "perf_low_lim",
                "performance",
            ],
        ]
